/*************************************************************************************
__MXSDOC__
Author:				Gavyn Thompson
Company:				GTVFX
Website:				www.gtvfx.com
Email:				gthompson@gtvfx.com
ScriptVersion:			v1.00
Updated:				12/31/2013
[Purpose]
Install for individual tool macro scripts.
__END__
*************************************************************************************/
struct macroInstall_lib
(
	self,
	ro,
	tempDir = ((getDir #temp)+@"\GTVFX"),
	scriptsDir = ((getDir #scripts)+@"\GTVFX\"),
	startupDir = ((getDir #userStartupScripts)+"\\"),
	macrosDir = ((getDir #userMacros)+"\\"),
	tempGTVFX = (tempDir + @"\Scripts\GTVFX"),
	sFileName = getFileNameFile (getFiles (tempGTVFX + "\\*.ms"))[1],
		
	fn fileIn_FN filePath =
	(
		if doesFileExist filePath then
		(
			fileIn filePath
		)
		else return false
		true
	),
	fn removeTempDir =
	(
		if doesFileExist tempDir then dosCommand ("rmdir \"" + tempDir + "\" /s /q")
	),
	fn copyFiles_FN fromDir toDir =
	(
		if not doesFileExist fromDir then
		(
			return false
		)
		else
		(		
			sFileArr = getFiles (fromDir + @"\*.ms")
			for i in sFileArr do
			(
				destFile = (toDir+(getFileNameFile i)+(getFileNameType i))
				if doesFileExist destFile then deleteFile destFile
				copyFile i destFile
			)
		)
		true
	),
	fn install_FN =
	(
		if not (makeDir scriptsDir) then return false
		if not copyFiles_FN tempGTVFX scriptsDir then return false
		true
	),
	
	fn installStartupScript =
	(
		if not copyFiles_FN tempGTVFX startupDir then return false
		fileIn_FN (startupDir + sFileName + ".ms")
	),
	
	fn uninstall_FN arr:#() =
	(
		if doesFileExist macrosDir then arr += getFiles (macrosDir + sFileName + "*")
		if doesFileExist scriptsDir then arr += getFiles (scriptsDir + sFileName + "*")
		if doesFileExist startupDir then arr += getFiles (startupDir + sFileName + "*")
		if arr.count != 0 then for i in arr do deleteFile i
	),
	fn formatMacro_FN macroName mcrCategory sFile stream =
	(
		format ("macroScript %\ncategory:\"%\"\nbuttonText:\"%\"\n(\n\tfileIn ((getDir #scripts)+\"\\\\GTVFX\\\\\"+\"%\"+\".ms\")\n)\n") \
		macroName mcrCategory macroName sFile to:stream
	),
	fn createMacro =
	(
		sFile = (getFiles (scriptsDir + sFileName + "*.ms"))[1]
		if matchPattern (getFileNameFIle sFile) pattern:"*GTVFX-LIB*" then
		(
			fileIn_FN sFile
		)
		else
		(
			fileName = substituteString (getFileNameFile sFile) "GTVFX-" ""	
			stream = stringStream ""
			formatMacro_FN fileName "GTVFX" sFileName stream
			seek stream 0
			readExpr stream
		)
	),
	fn ui =
	(
		rollout ro "GTVFX: Install" width:250
		(
			local self
			local self
			local dnToolTip
			local clrWindow = ((colorMan.getColor #window)*255)
			local clrText = ((colorMan.getColor #text)*255)
			local ClrBackGround = ((colorMan.getColor #background)*255)
			dotNetControl dNbtn_install "button" width:(ro.width-25) height:40
			dotNetControl dNbtn_unInstall "button" width:(ro.width-25) height:30
			hyperLink hyp_website "www.gtvfx.com" color:orange  hoverColor:red visitedColor:orange address:"http://www.gtvfx.com"
			
			fn initToolTip dNetObj caption =
			(
				if dnTooltip == undefined then
				(
					dnToolTip = dotnetobject "ToolTip"
					dnToolTip.AutoPopDelay = 5000
					dnToolTip.InitialDelay = 300
					dnToolTip.ReshowDelay = 300
					dnToolTip.ShowAlways = true
					dnToolTip.IsBalloon = true
				)
				dnToolTip.SetToolTip dNetObj caption
			)
			fn setDotNetWidget dNobj caption fontSize =
			(
				dNobj.text = caption
				dNobj.forecolor = dNobj.forecolor.FromArgb clrText.x clrText.y clrText.z
				dNobj.backColor = dNobj.backcolor.FromArgb ClrBackGround.x ClrBackGround.y ClrBackGround.z
				dNobj.Font = dotNetObject "System.Drawing.Font" "Segoe UI" fontSize ((dotNetClass "System.Drawing.FontStyle").bold)
			)
			fn initDnetBtn dNbtn caption fontSize style:#popup tooltip:"" = 
			(
				case style of
				(
					#flat:(dNbtn.flatStyle = dNbtn.flatStyle.flat)
					#popup:(dNbtn.flatStyle = dNbtn.flatStyle.popup)
					#system:(dNbtn.flatStyle = dNbtn.flatStyle.system)
				)
				setDotNetWidget dNbtn caption fontSize
				initToolTip dNbtn tooltip
			)
			fn posHyperLink posX posY =
			(
				hyp_website.pos = [posX,posY]
			)
			fn _init pself=
			(
				self = pself
				initDnetBtn dNbtn_install "Install Macro" 11 style:#popup
				initDnetBtn dNbtn_unInstall "Uninstall Macro" 11 style:#popup
			)
			on dNbtn_install mouseClick do
			(
				if matchPattern self.sFileName pattern:"*-LIB_*" then
				(
					self.installStartupScript()
				)
				else if self.install_FN() then
				(
					self.createMacro()
					self.removeTempDir()
					messageBox "GTVFX macroscript successfully installed.\nYou may need to restart Max if you are updating to a newer version of the macro." title:"GTVFX: Success :)"
				)
				else messageBox "Could not install the tool.\nDouble check that you have permissions to copy files to the 3ds Max root directory." title:"GTVFX: Error :("
				try(destroyDialog self.ro)catch() 
			)
			on dNbtn_unInstall mouseClick do
			(
				if queryBox "Are you sure you want to remove the macroscript? :(" then
				(
					self.uninstall_FN()
					self.removeTempDir()
					try(destroyDialog self.ro)catch()
					messageBox "Macroscript removed." title:"GTVFX: :("
				)
			)
			on ro open do
			(
				print ro.height
				posHyperLink (ro.width/2-40) (ro.height - 20)
			)
		)
		createDialog ro 
	),
	fn _init =
	(
		self = this
		ui()
		ro._init self
	),
	init = _init()
)
macroInit = macroInstall_lib()